#!/bin/bash

if [ $# = 0 ]; then
  echo "USAGE: all.sh config_file [sge_param]"
  exit 1
fi


# configuration and resubmiting of the script to SGE ---------------------------------------------------

conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

if [ ! -x "$conf" ] ; then
  echo "ERROR: all.sh: The configuration file not found: '$conf'"
  exit 1
fi
if [ ! -x "$conf_global" ] ; then
  echo "ERROR: all.sh The global configuration file not found: '$conf_global'"
  exit 1
fi

source $conf_global
source $conf

# add full path to the configuration file name if it does not exist (for SGE)
if [ ${conf:0:1} != "/" ] && [ ${conf:0:1} != "." ] ; then
  conf="`pwd`/$conf"
  conf_global="`pwd`/$conf_global"
fi

# run this script again in SGE (if the feature is enabled)
if [ "$use_sge" = "1" ] && [ "$2" != "nosge" ] ; then
  echo "Submitting script ${script_dir}/all.sh $conf to SGE..."
  sge_params="$3"
  ${script_dir}/manage_task.sh $conf "${script_dir}/all.sh $conf nosge" "std" $sge_params $global_sge_params
  exit 0
fi

# add the iteration part to the output directory name
out_dir=`echo "$out_dir" | awk --posix '/\/it[0-9]{3}*/ {print; next;} {print $1 "/it001";}'`
mkdir -p $out_dir

# parameterization --------------------------------------------------------------------------------------

if [ "$do_param_train" = "1" ] ; then
  mkdir -p $fea_dir
  mkdir -p $out_dir/lists
  # raw-fea file list  | filterflist.sh selects just nonexisting files
  awk '{f = $0; gsub("\\/", " "); gsub("\\.[^.]*$", "", $NF); print f, "'$fea_dir'/" $NF ".fea"}' $train_raw_list > $out_dir/lists/train_raw_fea.scp
  ${script_dir}/misc/filterflist.sh $out_dir/lists/train_raw_fea.scp 2 0 > $out_dir/lists/train_raw_fea_flt.scp
  # parameterization
  echo "Parametrize the train set ..."
  ${fea_script} $conf $out_dir/lists/train_raw_fea_flt.scp
  # list of features
  mkdir -p ${train_fea_list%/*}     
  awk '{print $2}' $out_dir/lists/train_raw_fea.scp > $train_fea_list
fi

if [ "$do_param_test" = "1" ] ; then
  mkdir -p $fea_dir
  mkdir -p $out_dir/lists
  # raw-fea file list
  awk '{f = $0; gsub("\\/", " "); gsub("\\.[^.]*$", "", $NF); print f, "'$fea_dir'/" $NF ".fea"}' $test_raw_list > $out_dir/lists/test_raw_fea.scp
  ${script_dir}/misc/filterflist.sh $out_dir/lists/test_raw_fea.scp 2 0 > $out_dir/lists/test_raw_fea_flt.scp
  # parameterization
  echo "Parametrize the test set ..."
  ${fea_script} $conf $out_dir/lists/test_raw_fea_flt.scp
  # list of features
  mkdir -p ${test_fea_list%/*}     
  awk '{print $2}' $out_dir/lists/test_raw_fea.scp > $test_fea_list
fi

if [ "$do_param_cv" = "1" ] ; then
  mkdir -p $fea_dir
  mkdir -p $out_dir/lists
  # raw-fea file list
  awk '{f = $0; gsub("\\/", " "); gsub("\\.[^.]*$", "", $NF); print f, "'$fea_dir'/" $NF ".fea"}' $cv_raw_list > $out_dir/lists/cv_raw_fea.scp
  ${script_dir}/misc/filterflist.sh $out_dir/lists/cv_raw_fea.scp 2 0 > $out_dir/lists/cv_raw_fea_flt.scp
  # parameterization
  echo "Parametrize the crossvalidation set ..."
  ${fea_script} $conf $out_dir/lists/cv_raw_fea_flt.scp
  # list of features  
  mkdir -p ${cv_fea_list%/*}     
  awk '{print $2}' $out_dir/lists/cv_raw_fea.scp > $cv_fea_list
fi



# verify if all feature files have enough frames for acoustic units, filter out short ones ------------

if  [ "$num_states" = "" ] ; then
  num_states=1
fi

mkdir -p $out_dir/lists
if [ ! -e $out_dir/lists/train_fea.scp ] || [ ! -e $out_dir/lists/test_fea.scp ] || [ ! -e $out_dir/lists/cv_fea.scp ] || [ ! -e $out_dir/lists/cv_fea.scp ] ; then
  # - number of models (phonemes/states) per segment (feature file)
  ${script_dir}/misc/mlf_nmodels.sh $ref_mlf > $out_dir/lists/models_per_seg.lst
  # - number of frames per segment (feature file)
  ${script_dir}/misc/fea_nframes.sh $train_fea_list > $out_dir/lists/train_fea_nfr.lst
  ${script_dir}/misc/fea_nframes.sh $test_fea_list > $out_dir/lists/test_fea_nfr.lst
  ${script_dir}/misc/fea_nframes.sh $cv_fea_list > $out_dir/lists/cv_fea_nfr.lst
  ${script_dir}/misc/fea_nframes.sh $lre_fea_list > $out_dir/lists/lre_fea_nfr.lst
  # - filter feature files - should have enough frames per acoustic units
  ${script_dir}/misc/fea_filter_p2u.sh $out_dir/lists/train_fea_nfr.lst $out_dir/lists/models_per_seg.lst $num_states > $out_dir/lists/train_fea.scp
  ${script_dir}/misc/fea_filter_p2u.sh $out_dir/lists/test_fea_nfr.lst $out_dir/lists/models_per_seg.lst $num_states > $out_dir/lists/test_fea.scp
  ${script_dir}/misc/fea_filter_p2u.sh $out_dir/lists/cv_fea_nfr.lst $out_dir/lists/models_per_seg.lst $num_states > $out_dir/lists/cv_fea.scp
  ${script_dir}/misc/fea_filter_p2u.sh $out_dir/lists/lre_fea_nfr.lst $out_dir/lists/models_per_seg.lst $num_states > $out_dir/lists/lre_fea.scp
fi


# split labels into more shorter labels (states) -------------------------------------------------------

if [ ! -e $out_dir/mlfs/ref.mlf ] || [ ! -e $out_dir/dicts/dict ] ; then
  mkdir -p $out_dir/mlfs
  mkdir -p $out_dir/dicts  
  if ( [ "$num_states" = "1" ] ) && [ "$num_states_file" = "" ] ; then
    ln -s $ref_mlf $out_dir/mlfs/ref.mlf
    ln -s $dict $out_dir/dicts/dict
  else
    echo "Splitting labels into states..."
    if [ "$num_states_file" = "" ] ; then
      ${script_dir}/misc/split_labels_mlf.pl -n $num_states $ref_mlf > $out_dir/mlfs/ref.mlf
      ${script_dir}/misc/split_labels_dict.pl -n $num_states $dict > $out_dir/dicts/dict
    else
      ${script_dir}/misc/split_labels_mlf.pl -f $num_states_file $ref_mlf > $out_dir/mlfs/ref.mlf
      ${script_dir}/misc/split_labels_dict.pl -f $num_states_file $dict > $out_dir/dicts/dict
    fi
  fi  
fi


# extract some additional parameters from files, create STK macros --------------------------------------

# extract parameters
mkdir -p $out_dir/config
if ! ${script_dir}/extract_params.sh $conf ; then
  exit 1
fi
source $out_dir/config/params.sh
  

# create feature transformations (macro files)
if ! ${script_dir}/create_macros.sh $conf ; then
  exit 1
fi

# train neural networks ---------------------------------------------------------------------------------
if [ "$do_train" = "1" ] ; then 
  echo
  echo "Training ..."
  
  # train streams (lower networks)
  if [ "$do_train_streams" = "1" ] ; then	
    echo "  - training streams ..."	
    if [ "$use_sge" = "1" ] && [ "$nstreams" != "1" ] ; then
      nstreams_1=$(( $nstreams - 1 ))
      jid=`${script_dir}/manage_task.sh $conf "${script_dir}/train_stream.sh $conf \\\$SGE_TASK_ID" train_streams "-t 1-$nstreams_1 -jid -m a" | tail -n 1`
      # run one stream on this computer
      if ! ${script_dir}/train_stream.sh $conf $nstreams ; then
        exit 1
      fi
      ${script_dir}/manage_task.sh $conf -waitfor $jid
    else
      for (( i=1 ; i <= $nstreams ; i=i+1 )) ; do
        if ! ${script_dir}/train_stream.sh $conf $i ; then
          exit 1
        fi
      done
    fi

    # join all macros for lower networks
    mkdir -p $out_dir/weights
    macro_file=$out_dir/weights/streams.ftr
    cat $out_dir/macros/input_mat.ftr > $macro_file
    for (( i=1 ; i <= nstreams ; i = i + 1 )) ; do
      if [ "${stream_nused_layers[$i]}" != "0" ] ; then
        cat $out_dir/norms/stream${i}_norm.ftr >> $macro_file 
        cat $out_dir/weights/stream$i/final/stream${i}_nn_cut.ftr >> $macro_file 
        cat $out_dir/macros/stream$i.ftr >> $macro_file 
      else
        cat $out_dir/macros/stream$i.ftr >> $macro_file 
      fi
    done
    cat $out_dir/macros/streams.ftr >> $macro_file

    # forward pass through all lower neural networks together
    macro_file=$out_dir/weights/streams_fwd.ftr
    echo "~o <VecSize> $nfeatures" > $macro_file
    cat $out_dir/weights/streams.ftr >> $macro_file
    cat $out_dir/macros/streams_fwd.ftr >> $macro_file

    # join macros for each lower neural netork for forward pass
    for (( i=1 ; i <= nstreams ; i = i + 1 )) ; do
      macro_file=$out_dir/weights/stream${i}_fwd.ftr    
      cat $out_dir/macros/input_mat.ftr > $macro_file
      if [ "${stream_nused_layers[$i]}" != "0" ] ; then
        cat $out_dir/norms/stream${i}_norm.ftr >> $macro_file 
        cat $out_dir/weights/stream$i/final/stream${i}_nn_cut.ftr >> $macro_file 
        cat $out_dir/macros/stream$i.ftr >> $macro_file 
      else
        cat $out_dir/macros/stream$i.ftr >> $macro_file 
      fi
      cat $out_dir/macros/stream${i}_fwd.ftr >> $macro_file
    done
  fi
  
  if [ "$do_train_merger" = "1" ] ; then
    echo "  - training merger ..."	
    
    ${script_dir}/train_merger.sh $conf
    
    #join all macros for merger network
    macro_file=$out_dir/weights/merger.ftr
    cat $out_dir/norms/merger_norm.ftr > $macro_file
    cat $out_dir/weights/merger/final/merger_nn.ftr | grep -v "<VecSize>" >> $macro_file
    cat $out_dir/macros/merger.ftr >> $macro_file
  fi
  
  # concatenate macros for lower networks and merger
  cat $out_dir/weights/streams.ftr > $out_dir/weights/full_sys.ftr
  cat $out_dir/weights/merger.ftr >> $out_dir/weights/full_sys.ftr
  cat $script_dir/templ/full_sys.ftr >> $out_dir/weights/full_sys.ftr
  
  echo "~o <VecSize> $nfeatures" > $out_dir/weights/full_sys_fwd.ftr
  cat $out_dir/weights/full_sys.ftr >> $out_dir/weights/full_sys_fwd.ftr
  cat $out_dir/macros/full_sys_fwd.ftr >> $out_dir/weights/full_sys_fwd.ftr
fi


# forward pass ------------------------------------------------------------------------------------------
echo
echo "Forwarding data ..."

if [ "$do_fwd_train" = "1" ] ; then
  if ! ${script_dir}/fwd.sh $conf $out_dir/lists/train_fea.scp $out_dir/posteriors ; then
    exit 1
  fi
fi 
if [ "$do_fwd_cv" = "1" ] ; then
  if ! ${script_dir}/fwd.sh $conf $out_dir/lists/cv_fea.scp $out_dir/posteriors ; then
    exit 1
  fi
fi 
if [ "$do_fwd_test" = "1" ] ; then
  if ! ${script_dir}/fwd.sh $conf $out_dir/lists/test_fea.scp $out_dir/posteriors ; then
    exit 1
  fi
fi 


# decoding ----------------------------------------------------------------------------------------------

mkdir -p $out_dir/rec
echo
echo "Decoding posteriors ..."

# create models, networks and tune phoneme insertion penalty
${script_dir}/prepare_decoder.sh $conf

# create lists
mkdir -p $out_dir/lists
awk -F '=' '{gsub("^.*/", "", $1); gsub("\\.[^.]*$", "", $1); print "'$out_dir'/posteriors/" $1 ".fea";}' $out_dir/lists/train_fea.scp > $out_dir/lists/train_post.scp
awk -F '=' '{gsub("^.*/", "", $1); gsub("\\.[^.]*$", "", $1); print "'$out_dir'/posteriors/" $1 ".fea";}' $out_dir/lists/cv_fea.scp > $out_dir/lists/cv_post.scp
awk -F '=' '{gsub("^.*/", "", $1); gsub("\\.[^.]*$", "", $1); print "'$out_dir'/posteriors/" $1 ".fea";}' $out_dir/lists/test_fea.scp > $out_dir/lists/test_post.scp

if [ "$do_decode_train" = "1" ] ; then
  ${script_dir}/decode.sh $conf $out_dir/lists/train_post.scp $out_dir/rec/train.mlf
fi
if [ "$do_decode_test" = "1" ] ; then
  ${script_dir}/decode.sh $conf $out_dir/lists/test_post.scp $out_dir/rec/test.mlf
fi
if [ "$do_decode_cv" = "1" ] ; then
  if [ -e "$out_dir/decoder/cv.mlf" ] ; then 
    cp $out_dir/decoder/cv.mlf $out_dir/rec/cv.mlf
   else
    ${script_dir}/decode.sh $conf $out_dir/lists/cv_post.scp $out_dir/rec/cv.mlf
  fi
fi


# scoring ------------------------------------------------------------------------------------------------

# backup previous results
mkdir -p $out_dir
if [ -e "$out_dir/results.txt"  ] ; then
	r=0
	while [  -e "$out_dir/results.old_$r"  ]  ;  do
		r=$(($r+1))
	done
	echo "results.txt - > results.old_$r"
	mv $out_dir/results.txt $out_dir/results.old_$r
fi

echo 
echo "Scoring ..."                                             | tee -a $out_dir/results.txt
echo "  Phoneme insertion penalty tuning list: $tune_fea_list" | tee -a $out_dir/results.txt
penalty=`tail -n 1 $out_dir/decoder/wip_tuning.txt`
echo "  Phoneme insertion penalty:             $penalty"       | tee -a $out_dir/results.txt
echo "  Language model scale:                  $lmscale"       | tee -a $out_dir/results.txt
echo "  Tune on maximum accuarcy:              $tune_max_acc"  | tee -a $out_dir/results.txt
echo

if [ "$do_score_test" = "1" ] && [ -e "$out_dir/rec/test.mlf" ] ; then
  echo >> $out_dir/results.txt
  echo "Test:"
  ${script_dir}/score_mlf.sh $conf $out_dir/rec/test.mlf       | tee -a $out_dir/results.txt | grep "WORD:"
fi
if [ "$do_score_cv" = "1" ] && [ -e $out_dir/rec/cv.mlf ] ; then
  echo >> $out_dir/results.txt
  echo "CV:"  
  ${script_dir}/score_mlf.sh $conf $out_dir/rec/cv.mlf         | tee -a $out_dir/results.txt | grep "WORD:"
fi
if [ "$do_score_train" = "1" ] && [ -e "$out_dir/rec/train.mlf" ] ; then
  echo "Train:"
  echo >> $out_dir/results.txt  
  ${script_dir}/score_mlf.sh $conf $out_dir/rec/train.mlf      | tee -a $out_dir/results.txt | grep "WORD:"
fi


# new force alignment ------------------------------------------------------------------------------------

echo
echo "Creating new force alignment ..."

# phoneme level MLF
${script_dir}/misc/mlf_st2phn.pl $out_dir/mlfs/ref.mlf > $out_dir/mlfs/ref_phn.mlf
mkdir -p $out_dir/falign
echo "#!MLF!#" > $out_dir/falign/ref_state.mlf

if [ "$do_falign_train" = "1" ] ; then
  ${script_dir}/falign.sh $conf $out_dir/lists/train_post.scp $out_dir/falign/train_full.mlf
  awk 'NF >= 3 {$4=""; $5=""; $6=""; gsub("\\[", "__", $3); gsub("\\]", "", $3); print; gsub(" +^", ""); next;} NR > 1 {print}' $out_dir/falign/train_full.mlf | awk '{gsub("\\.[^.]*\"$", ".lab\"", $1); print;}' >> $out_dir/falign/ref_state.mlf
fi
if [ "$do_falign_test" = "1" ] ; then
  ${script_dir}/falign.sh $conf $out_dir/lists/test_post.scp $out_dir/falign/test_full.mlf
  awk 'NF >= 3 {$4=""; $5=""; $6=""; gsub("\\[", "__", $3); gsub("\\]", "", $3); print; gsub(" +^", ""); next;} NR > 1 {print}' $out_dir/falign/test_full.mlf | awk '{gsub("\\.[^.]*\"$", ".lab\"", $1); print;}' >> $out_dir/falign/ref_state.mlf
fi
if [ "$do_falign_cv" = "1" ] ; then
  ${script_dir}/falign.sh $conf $out_dir/lists/cv_post.scp $out_dir/falign/cv_full.mlf
  awk 'NF >= 3 {$4=""; $5=""; $6=""; gsub("\\[", "__", $3); gsub("\\]", "", $3); print; gsub(" +^", ""); next;} NR > 1 {print}' $out_dir/falign/cv_full.mlf | awk '{gsub("\\.[^.]*\"$", ".lab\"", $1); print;}' >> $out_dir/falign/ref_state.mlf
fi

multistate=`awk 'BEGIN {multist = 0}; $3 != "" {a = gensub(".*(__[0-9])+", "\\\1", "g"); if(a != prev_a && prev_a != "") multist = 1; prev_a = a;} END {print multist;}' $out_dir/falign/ref_state.mlf`
if [ "$multistate" = "1" ] ; then
  cp $out_dir/falign/ref_state.mlf $out_dir/falign/ref.mlf
else
  awk '{gsub("__[0-9]+$", "", $3); print}' $out_dir/falign/ref_state.mlf > $out_dir/falign/ref.mlf   # state to phoneme labels
fi

# prepare new iteraction ---------------------------------------------------------------------------------

new_dir=`echo "$out_dir" | awk '{it =  substr($0, length($0) - 2, 3); path = substr($0, 1, length($0) - 3); it++; printf "%s%03d\n", path, it;}'`

if [ "$do_prepare_new_it" = "1" ] ; then
  echo
  echo "Preparing new iteration ..."

#  iteration=`echo "$out_dir" | awk '{print substr($0, length($0) - 2, 3) + 0;}'`

  mkdir -p $new_dir/inputs/mlfs
  mkdir -p $new_dir/inputs/lists
  mkdir -p $new_dir/inputs/dicts
  mkdir -p $new_dir/scripts

  rm -f $new_dir/inputs/mlfs/ref.mlf
  rm -f $new_dir/inputs/lists/train.scp
  rm -f $new_dir/inputs/lists/test.scp
  rm -f $new_dir/inputs/lists/cv.scp
  rm -f $new_dir/inputs/lists/lre.scp
  rm -f $new_dir/inputs/lists/tune.scp
  rm -f $new_dir/inputs/dicts/dict
  rm -f $new_dir/scripts/_global.sh
  rm -f $new_dir/scripts/run.sh

  ln -s $out_dir/falign/ref.mlf $new_dir/inputs/mlfs
  ln -s $out_dir/lists/train_fea.scp $new_dir/inputs/lists/train.scp
  ln -s $out_dir/lists/test_fea.scp $new_dir/inputs/lists/test.scp
  ln -s $out_dir/lists/cv_fea.scp $new_dir/inputs/lists/cv.scp
  ln -s $out_dir/lists/lre_fea.scp $new_dir/inputs/lists/lre.scp
  ln -s $out_dir/lists/cv_fea.scp $new_dir/inputs/lists/tune.scp    # this should be fixed
  cp $out_dir/dicts/dict $new_dir/inputs/dicts
  ln -s $conf_global $new_dir/scripts

  awk '/^ *out_dir=/         {next;}
       /^ *ref_mlf=/         {next;}
       /^ *dict=/            {next;}
       /^ *train_fea_list=/  {next;}
       /^ *test_fea_list=/   {next;}
       /^ *cv_fea_list=/     {next;}
       /^ *lre_fea_list=/    {next;}
       /^ *tune_fea_list=/   {next;}
       /^ *num_states=/      {next;}
       /^ *do_param_train=/  {next;}
       /^ *do_param_cv=/     {next;}
       /^ *do_param_test=/   {next;}
       /# These lines were/  {next;}
                             {print;}
       END                   {
                               print "\n# These lines were generated automatically!";
                               print "out_dir='$new_dir'";
                               print "ref_mlf='$new_dir'/inputs/mlfs/ref.mlf";
                               print "dict='$new_dir'/inputs/dicts/dict";
                               print "train_fea_list='$new_dir'/inputs/lists/train.scp";
                               print "test_fea_list='$new_dir'/inputs/lists/test.scp";
                               print "cv_fea_list='$new_dir'/inputs/lists/cv.scp";
                               print "lre_fea_list='$new_dir'/inputs/lists/lre.scp";
                               print "tune_fea_list='$new_dir'/inputs/lists/tune.scp";
                               print "num_states=1";
                               print "do_param_train=0";
                               print "do_param_cv=0";
                               print "do_param_test=0";
                             }' $conf > $new_dir/scripts/run.sh

  chmod ug+x $new_dir/scripts/_global.sh
  chmod ug+x $new_dir/scripts/run.sh
fi

# run new iteration --------------------------------------------------------------------------------------
if [ "$do_run_new_it" = "1" ] && [ "$num_iters" != "" ] ; then
  echo
  echo "Running new iteration ..."
  iteration=`echo "$out_dir" | awk '{print substr($0, length($0) - 2, 3) + 0;}'`
  if [[ "$iteration" -lt "$num_iters" ]] ; then
     if [ "$use_sge" = "1" ] ; then
       ${script_dir}/manage_task.sh $new_dir/scripts/run.sh "${script_dir}/all.sh $new_dir/scripts/run.sh nosge" "std" $sge_params $global_sge_params
     else
       ${script_dir}/all.sh $new_dir/scripts/run.sh 
     fi
  fi
fi

# clean output directory ---------------------------------------------------------------------------------
#${script_dir}/misc/rm_workdir.sh $conf $main_work_dir
#find $out_dir -type d -empty -exec rmdir {} \; 2> /dev/null
