#!/bin/sh

# base directory for scripts, experiments etc.
base_dir="/mnt/matylda4/schwarzp/EXP/TR_SVN3/NNScripts"

# directory with executables (STK, HTK ...)
bin_dir="$base_dir/tools"

# directory with scripts
script_dir="$base_dir/scripts"

# temporal directory on local machines
tmp_dir="/tmp/$USER"

# use the temporal directory on local machines
use_tmp=1

# Minimal free space in temporal directory on local machine. If there is not enough space, 
# the experimental directory is used instead
tmp_min_space=15000

# use Sun Grid Engine (SGE)
use_sge=0

# SGE queue (selected computers)
sge_computers="all.q@@stable,all.q@@speech"

# SGE queue - fast computers
# This queue is used if manage_task.sh is run with the '-fast' switch 
# (this feature is not currently used by other scripts). 
sge_fast_computers=$sge_computers

# requested resources for SGE
sge_resource="tmp_free=5000,disk_free=5000"

# an email for notification when a job is done of aborted
sge_email=$USER@fit.vutbr.cz

# an interval for phoneme insertion penalty tuning
penalty_interval="-20 20"

# default phoneme insertion penalty
default_penalty=-4

# Tune phoneme insertion penalty for maximal accuracy,
# else the recognizer is tuned for equal number of insertion and deletion errors.
# The tuning is done on the cross-validation set.
tune_max_acc=1

# default language model scale factor
lmscale=1

# stages to run
# - parameterize the train, test and crossvalidation set
do_tr_param=1
do_te_param=1
do_cv_param=1

# - train the system
do_train=1
# - train lower (stream) networks
do_train_streams=1
# - train merger
do_train_merger=1

# - forward pass trough the system (hierarchical structure of neural networks)
do_fwd_train=1
do_fwd_cv=1
do_fwd_test=1

# - decode posteriors and produce phoneme strings
do_decode_train=1
do_decode_cv=1
do_decode_test=1

# - scoring (phoneme error rate and confusion matrix)
do_score_train=1
do_score_cv=1
do_score_test=1

# - force align the train, test and cross-validation set, create new labels
do_falign_train=1
do_falign_test=1
do_falign_cv=1

# - prepare new iteration of experiment (experiment = training, forward pass of data 
#   through the system, scoring, realignment): 1) create a new subdirectory 'itXXX' 
#   in the experimental directory with new labels and all necessary files linked;
#   2) create a new configuration file 'itXXX/scripts/run.sh'
do_prepare_new_it=1
# - run new iteration (run 'all.sh' with the 'run.sh' as a configuration file; directly or through SGE)
do_run_new_it=1

# default parameterization script - 23 Mel-banks extracted from 16kHz speech
fea_script=$base_dir/scripts/param/hcopy23mel_16k_0.sh

# number of performed iteration s
num_iters=1
 
# Number of states per unit (phoneme) in the HMM model. If this number is greater than 1, 
# the acoustic units are spit uniformly to states.
num_states=1

# File with numbers of states per acoustic units (phonemes). The format is "label count" per line.
# This can be used if numbers of states differ.
num_states_file=""

# trace options
# - trace ofption for mean and variance normalization estimation before of lower neural networks (parameter for SERest -T option)
trace_streams_normest=1

# NN learning scheduler. Neural networks are trained in epochs.
# - the algorithm divides the learning rate by 2 if the increment in accuacy (compared to the previous epoch)
#   evaluated on the feature files given by the 'lre_fea_list' list is smaller then this value
ts_delta_dec=0.5
# - it is enough to satisfy the condition above once and the algorithm divides the learning rate by 2 in all following epochs
ts_keep_dec=1
# - the algorithm stops training if increment in accuracy evaluated on the feature files given by the 'cv_fea_list'
#   list is smaller then this value
ts_deta_stop=0.5
# - minimal and maximal number of epochs of NN learning
ts_min_iter=1
ts_max_iter=9999

# Cache the output of lower networks on hard drive for training of the merger network, else
# the output is calculated on-line for each epoch during training. The training is faster when the output is small and cached.
# If the output is big, the chached files are very big. It is faster to calculate it on-line then.
save_streams_output=1

# keep temporal files (outputs of lower neural networks)
keep_tmp_files=0

# locales setting - to prevent errors caused by different decimal point separators in different languages (EN '.',  CZ ',')
LANG=en_US
LC_ALL=C
