#!/bin/sh

# experimental directory
exp_dir="TIMIT_LCRC_DCT11_3st"

# output directory
out_dir="/mnt/scratch02/schwarzp/TIMIT/Experiments/$exp_dir/it001"

# parameterize speech files
do_param_train=1
do_param_cv=1
do_param_test=1

# train the system
do_train=1

# forward pass trough the networks
do_fwd_train=1
do_fwd_cv=1
do_fwd_test=1

# decode posteriors and produce strings
do_decode_train=1
do_decode_cv=1
do_decode_test=1

# scoring
do_score_train=1
do_score_cv=1
do_score_test=1


# input directory with lists, reference labels and dictionary
input_dir=/mnt/scratch02/schwarzp/TIMIT/timit-16k-uniq

# lists of waveform files
train_raw_list=${input_dir}/lists/train_raw.scp
test_raw_list=${input_dir}/lists/test_raw.scp
cv_raw_list=${input_dir}/lists/cv_raw.scp

# reference mlf
ref_mlf=${input_dir}/mlfs/ref.mlf

# the dictionary
dict=${input_dir}/dicts/dict

# directory for preparametrized speech files
fea_dir=${out_dir}/features

# parametrization script (mel bank generation script)
fea_script=${base_dir}/scripts/param/hcopy23mel_16k_0.sh

# lists of melbanks files
# - a directory for lists
fea_lists_dir=${out_dir}/lists
# - training, test and cross-validation list
train_fea_list=${fea_lists_dir}/train.scp
test_fea_list=${fea_lists_dir}/test.scp
cv_fea_list=${fea_lists_dir}/cv.scp
# - a list for estimation of NN learning rate; the set of files must be a subset of files contained 
#   in the train, test or cross-validation set; if the feature file lists are generated automaticaly (do_param_{train,cv,test}=1),
#   it must be the equal to the train_fea_list or cv_fea_list or test_fea_list
lre_fea_list=${fea_lists_dir}/train.scp
# - a list for word insertion penalty tuning; the above constrains apply too.
tune_fea_list=${fea_lists_dir}/cv.scp

# number of iterations (training, forward passs, decoding, scoring, force alignment)
num_iters=5

# number of states for each phoneme
num_states=3

# minimal number of epochs for neural network training
ts_min_iter=5

# use Sun Grid Engine
use_sge=1


# === streams ===

# - repeate the first frame N times
streams_start_frame_ext=15
# - repeate the last frame N times
streams_end_frame_ext=15

# - Input patterns for each network. This pattern is applied on a time-spectral block. The time span is given by 
#   streams_start_frame_ext+streams_end_frame_ext+1. The spectral span is given by bumber of Mel-banks. The definition
#   allows multiple patterns separated by space. Everything in [] are predefined values and the values are calculated 
#   and substituted automatically.
stream_ipattern[1]="1:[NFEATURES],1:16"
stream_ipattern[2]="1:[NFEATURES],16:31"

# - Input windows. See description of STK macros for details of definition. 
#   The discrete window can be applied using the '<Window>' macro.
stream_iwindow[1]="<WindowPredef> [INPUT_SIZE] LinSpace 0.01 1 [NFEATURES]"
stream_iwindow[2]="<WindowPredef> [INPUT_SIZE] LinSpace 1 0.01 [NFEATURES]"

# - Input transform. See description of STK macros for details of definition.
#   The shortcut 'Nx'  copies the transform N times in parallel. It is expanded to '<NumBlocks> N <Block> 1 transform <Block> 2 transform ...'
stream_itransform[1]="[NFEATURES]x <XformPredef> 11 16 DCT 1"
stream_itransform[2]="[NFEATURES]x <XformPredef> 11 16 DCT 1"

# - Definition of neural networks. All weights, biases and nonlinearity have their own layers. The layers are separated by ';'. 
#   The definition is expanded to a serial STK macro '<NumLayers> N <Layer> 1 layer_definition1 <Layer> 2 layer_definition2 ...'
stream_net[1]="<XformPredef> 500 [INPUT_SIZE] Random -0.1 0.1 999; <BiasPredef> 500 Random -4.1 -3.9 999; <Sigmoid> 500;
               <XformPredef> [OUTPUT_SIZE] 500 Random -0.1 0.1 999; <BiasPredef> [OUTPUT_SIZE] Random -4.1 -3.9 999; <SoftMax> [OUTPUT_SIZE]"        
stream_net[2]="<XformPredef> 500 [INPUT_SIZE] Random -0.1 0.1 999; <BiasPredef> 500 Random -4.1 -3.9 999; <Sigmoid> 500;
               <XformPredef> [OUTPUT_SIZE] 500 Random -0.1 0.1 999; <BiasPredef> [OUTPUT_SIZE] Random -4.1 -3.9 999; <SoftMax> [OUTPUT_SIZE]"

# - Number of used layers for forward pass. Not all layers must be used. This allow for example to generate bottle neck features for LVCSR
stream_nused_layers[1]=999
stream_nused_layers[2]=999

# - outout transfrom
stream_otransform[1]="<WindowPredef> [INPUT_SIZE] Const 1"
stream_otransform[2]="<WindowPredef> [INPUT_SIZE] Const 1"

# - output (softening) function
stream_ofunc[1]="<Log> [SIZE]"
stream_ofunc[2]="<Log> [SIZE]"

# - initial learning rate
streams_learning_rate=0.008


# === merger ===

merger_start_frame_ext=$streams_start_frame_ext
merger_end_frame_ext=$streams_start_frame_ext

merger_ifunc="<WindowPredef> [SIZE] Const 1"

merger_ipattern="1:[MERGER_NFEATURES],1"

merger_iwindow="<WindowPredef> [INPUT_SIZE] Const 1"

merger_itransform="<WindowPredef> [INPUT_SIZE] Const 1"

merger_net="<XformPredef> 500 [INPUT_SIZE] Random -0.1 0.1 999; <BiasPredef> 500 Random -4.1 -3.9 999; <Sigmoid> 500;
            <XformPredef> [OUTPUT_SIZE] 500 Random -0.1 0.1 999; <BiasPredef> [OUTPUT_SIZE] Random -4.1 -3.9 999; <SoftMax> [OUTPUT_SIZE]"

merger_ofunc="<Log> [SIZE]"

merger_learning_rate=0.008

