#!/bin/sh

if [ $# -lt 1 ]; then
	echo "USAGE: create_macros.sh config_file"
	exit 1
fi

conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

out_dir=`echo "$out_dir" | awk --posix '/\/it[0-9]{3}*/ {print; next;} {print $1 "/it001";}'`
source $out_dir/config/params.sh

echo
echo "Creating feature transformation files (macro files) ..."

mkdir -p $out_dir/macros

# input matrix - stacking, transposition
awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
      gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
      gsub("\\[NSTREAMS\\]", "'$nstreams'"); print;}' ${script_dir}/templ/input_mat.ftr > $out_dir/macros/input_mat.ftr


for (( i=1 ; i <= nstreams ; i = i + 1 )) ; do
  if [ "${stream_nused_layers[$i]}" != "0" ] ; then
    # transformations for estimation of normalization coefs for streams neural networks
    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
          gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
          gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
          gsub("\\[STREAM_NBLOCKS\\]", "'${stream_nblocks[$i]}'");
          gsub("\\[STREAM_IPATTERN\\]", "'"${stream_ipattern_sub[$i]}"'"); 
          gsub("\\[STREAM_INPUT_WINDOW\\]", "'"${stream_iwindow_sub[$i]}"'"); 
          gsub("\\[STREAM_INPUT_TRANSFORM\\]", "'"${stream_itransform_sub[$i]}"'"); 
          gsub("\\[STREAM_NET_ISIZE\\]", "'"${stream_net_isize[$i]}"'");
          print;}' ${script_dir}/templ/stream_train_norm.ftr > $out_dir/macros/stream${i}_train_norm.ftr
    
    # hmm for extimation of normalization coefs for streams neural networks
    mean=""
    var=""
    for (( j=0 ; j<stream_net_isize[$i] ; j=j+1  )) ; do
      mean="$mean 0"
      var="$var 1"
    done
    awk '{gsub("\\[NET_ISIZE\\]", "'"${stream_net_isize[$i]}"'");
          gsub("\\[MEAN_VALS\\]", "'"$mean"'");
          gsub("\\[VARIANCE_VALS\\]", "'"$var"'");
	  print;}' ${script_dir}/templ/norm_est.hmmdefs > $out_dir/macros/stream${i}_norm_est.hmmdefs

    # transformations for NN training
    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
          gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
          gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
          gsub("\\[NCLASSES\\]", "'$nclasses'"); 
          gsub("\\[STREAM_NBLOCKS\\]", "'${stream_nblocks[$i]}'");
          gsub("\\[STREAM_IPATTERN\\]", "'"${stream_ipattern_sub[$i]}"'"); 
          gsub("\\[STREAM_INPUT_WINDOW\\]", "'"${stream_iwindow_sub[$i]}"'"); 
          gsub("\\[STREAM_INPUT_TRANSFORM\\]", "'"${stream_itransform_sub[$i]}"'"); 
          gsub("\\[STREAM_NET_ISIZE\\]", "'"${stream_net_isize[$i]}"'");
          gsub("\\[STREAM_NAME\\]", "'"stream${i}"'");
          print;}' ${script_dir}/templ/stream_train_nn.ftr > $out_dir/macros/stream${i}_train_nn.ftr  

    # neural networks
    net=`echo "${stream_net_sub[$i]}" | awk '{print "    <Layer> " NR "\n      " $0;}' | tr '\n' ';'`   # tr - awk does not like new lines in strings  
    awk '{
         gsub("\\[NET_NAME\\]", "stream'$i'_nn");
         gsub("\\[NET_NLAYERS\\]", "'"${stream_net_nlayers[$i]}"'");
         gsub("\\[NET_LAYERS\\]", "'"$net"'");
         print;}' ${script_dir}/templ/nn.ftr | tr ';' '\n' > $out_dir/macros/stream${i}_nn.ftr       
    
    # stream forward pass macros
    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
         gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
         gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
         gsub("\\[NCLASSES\\]", "'$nclasses'"); 
         gsub("\\[STREAM_NBLOCKS\\]", "'${stream_nblocks[$i]}'");
         gsub("\\[STREAM_IPATTERN\\]", "'"${stream_ipattern_sub[$i]}"'"); 
	       gsub("\\[STREAM_INPUT_WINDOW\\]", "'"${stream_iwindow_sub[$i]}"'"); 
	       gsub("\\[STREAM_INPUT_TRANSFORM\\]", "'"${stream_itransform_sub[$i]}"'"); 
	       gsub("\\[STREAM_OUTPUT_TRANSFORM\\]", "'"${stream_otransform_sub[$i]}"'"); 	  
	       gsub("\\[STREAM_OUTPUT_FUNC\\]", "'"${stream_ofunc_sub[$i]}"'"); 	  	  
	       gsub("\\[STREAM_NET_ISIZE\\]", "'"${stream_net_isize[$i]}"'");
	       gsub("\\[STREAM_NAME\\]", "'"stream${i}"'");
	       print;}' ${script_dir}/templ/stream.ftr > $out_dir/macros/stream${i}.ftr  
  else
    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
         gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
         gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
         gsub("\\[NCLASSES\\]", "'$nclasses'"); 
	       gsub("\\[STREAM_NBLOCKS\\]", "'${stream_nblocks[$i]}'");
	       gsub("\\[STREAM_IPATTERN\\]", "'"${stream_ipattern_sub[$i]}"'"); 
	       gsub("\\[STREAM_INPUT_WINDOW\\]", "'"${stream_iwindow_sub[$i]}"'"); 
	       gsub("\\[STREAM_INPUT_TRANSFORM\\]", "'"${stream_itransform_sub[$i]}"'"); 
	       gsub("\\[STREAM_OUTPUT_TRANSFORM\\]", "'"${stream_otransform_sub[$i]}"'"); 	  
	       gsub("\\[STREAM_OUTPUT_FUNC\\]", "'"${stream_ofunc_sub[$i]}"'"); 	  	  
	       gsub("\\[STREAM_NET_ISIZE\\]", "'"${stream_net_isize[$i]}"'");
	       gsub("\\[STREAM_NAME\\]", "'"stream${i}"'");
	       print;}' ${script_dir}/templ/stream_nonn.ftr > $out_dir/macros/stream${i}.ftr  	  	       
  fi	
  
  awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
       gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
       gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
       gsub("\\[NCLASSES\\]", "'$nclasses'"); 
	     gsub("\\[STREAM_NBLOCKS\\]", "'${stream_nblocks[$i]}'");
	     gsub("\\[STREAM_IPATTERN\\]", "'"${stream_ipattern_sub[$i]}"'"); 
	     gsub("\\[STREAM_INPUT_WINDOW\\]", "'"${stream_iwindow_sub[$i]}"'"); 
	     gsub("\\[STREAM_INPUT_TRANSFORM\\]", "'"${stream_itransform_sub[$i]}"'"); 
	     gsub("\\[STREAM_OUTPUT_TRANSFORM\\]", "'"${stream_otransform_sub[$i]}"'"); 	  
	     gsub("\\[STREAM_OUTPUT_FUNC\\]", "'"${stream_ofunc_sub[$i]}"'"); 	  	  
	     gsub("\\[STREAM_NET_ISIZE\\]", "'"${stream_net_isize[$i]}"'");
	     gsub("\\[STREAM_NAME\\]", "'"stream${i}"'");
	     print;}' ${script_dir}/templ/stream_fwd.ftr > $out_dir/macros/stream${i}_fwd.ftr  
done

# forward pass macro for streams
ipatterns=""
streams=""
for (( i=1 ; i <= nstreams ; i = i + 1 )) ; do
  if [ "$ipatterns" = "" ] ; then
    ipatterns="${stream_ipattern_sub[$i]}"
    streams="<Block> $i ~x ^stream$i^"
  else
    ipatterns="$ipatterns ${stream_ipattern_sub[$i]}"
    streams="$streams; <Block> $i ~x ^stream$i^"
  fi
done

awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
      gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
      gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
      gsub("\\[NCLASSES\\]", "'$nclasses'"); 
	    gsub("\\[STREAM_NBLOCKS\\]", "'${stream_nblocks[$i]}'");
	    gsub("\\[STREAM_IPATTERN\\]", "'"${stream_ipattern_sub[$i]}"'"); 
	    gsub("\\[STREAM_INPUT_WINDOW\\]", "'"${stream_iwindow_sub[$i]}"'"); 
	    gsub("\\[STREAM_INPUT_TRANSFORM\\]", "'"${stream_itransform_sub[$i]}"'"); 
	    gsub("\\[STREAM_OUTPUT_TRANSFORM\\]", "'"${stream_otransform_sub[$i]}"'"); 	  
	    gsub("\\[STREAM_OUTPUT_FUNC\\]", "'"${stream_ofunc_sub[$i]}"'"); 	  	  
	    gsub("\\[STREAM_NET_ISIZE\\]", "'"${stream_net_isize[$i]}"'");
	    gsub("\\[STREAM_NAME\\]", "'"stream${i}"'");
	    gsub("\\[NBLOCKS\\]", "'"$streams_total_nblocks"'");
	    gsub("\\[IPATTERNS\\]", "'"$ipatterns"'");	    
	    gsub("\\[STREAMS\\]", "'"$streams"'");	    
	    print;}' ${script_dir}/templ/streams.ftr | tr ';^' '\n"' > $out_dir/macros/streams.ftr  	  

     # macro for forward pass through all lower networks
    awk '{gsub("\\[NFEATURES\\]", "'$nfeatures'");           
          gsub("\\[NCLASSES\\]", "'$nclasses'");    
          gsub("\\[MERGER_NFEATURES\\]", "'$merger_isize'");                  
          print;}' ${script_dir}/templ/streams_fwd.ftr > $out_dir/macros/streams_fwd.ftr     

    # transformations for estimation of normalization coefs for merger neural network
    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
          gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
          gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
          gsub("\\[MERGER_NET_ISIZE\\]", "'$merger_net_isize'");
          gsub("\\[MERGER_CONTEXT\\]", "'$merger_context'"); 
          gsub("\\[MERGER_NBLOCKS\\]", "'$merger_nblocks'"); 
          gsub("\\[MERGER_NFEATURES\\]", "'$merger_isize'"); 
          gsub("\\[MERGER_IPATTERN\\]", "'"$merger_ipattern_sub"'"); 
          gsub("\\[MERGER_INPUT_WINDOW\\]", "'"$merger_iwindow_sub"'");
          gsub("\\[MERGER_INPUT_TRANSFORM\\]", "'"$merger_itransform_sub"'");
          print;}' ${script_dir}/templ/merger_train_norm.ftr > $out_dir/macros/merger_train_norm.ftr

    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
          gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
          gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
          gsub("\\[MERGER_NET_ISIZE\\]", "'$merger_net_isize'");
          gsub("\\[MERGER_CONTEXT\\]", "'$merger_context'"); 
          gsub("\\[MERGER_NBLOCKS\\]", "'$merger_nblocks'"); 
          gsub("\\[MERGER_NFEATURES\\]", "'$merger_isize'"); 
          gsub("\\[MERGER_IPATTERN\\]", "'"$merger_ipattern_sub"'"); 
          gsub("\\[MERGER_INPUT_WINDOW\\]", "'"$merger_iwindow_sub"'");
          gsub("\\[MERGER_INPUT_TRANSFORM\\]", "'"$merger_itransform_sub"'");
          print;}' ${script_dir}/templ/merger_train_norm_posti.ftr > $out_dir/macros/merger_train_norm_posti.ftr

    # hmm for extimation of normalization coefs for streams neural networks
    mean=""
    var=""
    for (( j=0 ; j < merger_net_isize ; j = j + 1  )) ; do
      mean="$mean 0"
      var="$var 1"
    done
    awk '{gsub("\\[NET_ISIZE\\]", "'"$merger_net_isize"'");
          gsub("\\[MEAN_VALS\\]", "'"$mean"'");
          gsub("\\[VARIANCE_VALS\\]", "'"$var"'");
	  print;}' ${script_dir}/templ/norm_est.hmmdefs > $out_dir/macros/merger_norm_est.hmmdefs

    # transformation for merger neural network training
    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
          gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
          gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
          gsub("\\[NCLASSES\\]", "'$nclasses'");           
          gsub("\\[MERGER_NET_ISIZE\\]", "'$merger_net_isize'");
          gsub("\\[MERGER_CONTEXT\\]", "'$merger_context'"); 
          gsub("\\[MERGER_NBLOCKS\\]", "'$merger_nblocks'"); 
          gsub("\\[MERGER_NFEATURES\\]", "'$merger_isize'"); 
          gsub("\\[MERGER_IPATTERN\\]", "'"$merger_ipattern_sub"'"); 
          gsub("\\[MERGER_INPUT_WINDOW\\]", "'"$merger_iwindow_sub"'");
          gsub("\\[MERGER_INPUT_TRANSFORM\\]", "'"$merger_itransform_sub"'");
          print;}' ${script_dir}/templ/merger_train_nn.ftr > $out_dir/macros/merger_train_nn.ftr

    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
          gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
          gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
          gsub("\\[NCLASSES\\]", "'$nclasses'");           
          gsub("\\[MERGER_NET_ISIZE\\]", "'$merger_net_isize'");
          gsub("\\[MERGER_CONTEXT\\]", "'$merger_context'"); 
          gsub("\\[MERGER_NBLOCKS\\]", "'$merger_nblocks'"); 
          gsub("\\[MERGER_NFEATURES\\]", "'$merger_isize'"); 
          gsub("\\[MERGER_IPATTERN\\]", "'"$merger_ipattern_sub"'"); 
          gsub("\\[MERGER_INPUT_WINDOW\\]", "'"$merger_iwindow_sub"'");
          gsub("\\[MERGER_INPUT_TRANSFORM\\]", "'"$merger_itransform_sub"'");
          print;}' ${script_dir}/templ/merger_train_nn_posti.ftr > $out_dir/macros/merger_train_nn_posti.ftr

    # merger neural network
    net=`echo "$merger_net_sub" | awk '{print "    <Layer> " NR "\n      " $0;}' | tr '\n' ';'`   # tr - awk does not like new lines in strings  
    awk '{
         gsub("\\[NET_NAME\\]", "merger_nn");
         gsub("\\[NET_NLAYERS\\]", "'$merger_net_nlayers'");
         gsub("\\[NET_LAYERS\\]", "'"$net"'");
         print;}' ${script_dir}/templ/nn.ftr | tr ';' '\n' > $out_dir/macros/merger_nn.ftr       

    # merger forward pass macro
    awk '{gsub("\\[CONTEXT\\]", "'$stream_context'");
          gsub("\\[NFEATURES\\]", "'$nfeatures'"); 
          gsub("\\[NSTREAMS\\]", "'$nstreams'"); 
          gsub("\\[NCLASSES\\]", "'$nclasses'");           
          gsub("\\[MERGER_NET_ISIZE\\]", "'$merger_net_isize'");
          gsub("\\[MERGER_CONTEXT\\]", "'$merger_context'"); 
          gsub("\\[MERGER_NBLOCKS\\]", "'$merger_nblocks'"); 
          gsub("\\[MERGER_NFEATURES\\]", "'$merger_isize'"); 
          gsub("\\[MERGER_IPATTERN\\]", "'"$merger_ipattern_sub"'"); 
          gsub("\\[MERGER_INPUT_WINDOW\\]", "'"$merger_iwindow_sub"'");
          gsub("\\[MERGER_INPUT_TRANSFORM\\]", "'"$merger_itransform_sub"'");
          gsub("\\[MERGER_OUTPUT_FUNC\\]", "'"$merger_ofunc_sub"'");
          print;}' ${script_dir}/templ/merger.ftr > $out_dir/macros/merger.ftr
          
     # macro for forward pass through the whole system
    awk '{gsub("\\[NFEATURES\\]", "'$nfeatures'");           
          gsub("\\[NCLASSES\\]", "'$nclasses'");           
          print;}' ${script_dir}/templ/full_sys_fwd.ftr > $out_dir/macros/full_sys_fwd.ftr     
 