#!/bin/sh

if [ $# -lt 1 ]; then
  echo "USAGE: extract_params.sh config_file"
  exit 1
fi

conf=$1
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf

# output directory - add iteration part
out_dir=`echo "$out_dir" | awk --posix '/\/it[0-9]{3}*/ {print; next;} {print $1 "/it001";}'`
mkdir -p $out_dir

out_param_file=$out_dir/config/params.sh
echo
echo "Extracting system configuration parameters ..."
echo "# These parameters were extracted from configuration files and feature files" > $out_param_file

echo "out_dir=$out_dir"             >> $out_param_file

# information about machine
main_work_dir=`${script_dir}/misc/get_workdir.sh $conf`
echo "  Main working directory:       $main_work_dir"
echo "main_work_dir=$main_work_dir" >> $out_param_file
machine=`uname -n`
echo "  Computer:                     $machine"
time=`date`
echo "  Time:                         $time"

# get number of features
nfeatures=0
fea_file=`head -n 1 $out_dir/lists/train_fea.scp | tr '=[' '  ' | cut -d ' ' -f 2`

if [ -e "$fea_file" ] ; then
  nfeatures=`${bin_dir}/HList -h $fea_file | awk 'NR==2 {print $3;}'`
else
  fea_file=`head -n 1 $out_dir/lists/test_fea.scp`
  if [ -e "$fea_file" ] ; then
    nfeatures=`${bin_dir}/HList -h $fea_file | awk 'NR==2 {print $3;}'`        
  else
    fea_file=`head -n 1 $out_dir/lists/cv_fea.scp`
    if [ -e "$fea_file" ] ; then
      nfeatures=`${bin_dir}/HList -h $fea_file | awk 'NR==2 {print $3;}'`    
    else
      echo "ERROR: Can not get number of input features, feature files are not found ($out_dir/lists/train_fea.scp)." > /dev/stderr
      exit 1
    fi
  fi
fi
nfeatures=$(( nfeatures / 4 ))
echo "  Number of features:           $nfeatures"
echo "nfeatures=$nfeatures" >> $out_param_file

# number of classes (labels or posteriors)
nclasses=`wc $out_dir/dicts/dict | awk '{print $1}'`
echo "  Number of classes:            $nclasses"
echo "nclasses=$nclasses" >> $out_param_file

# temporal context
stream_context=$(( streams_start_frame_ext + streams_end_frame_ext + 1 ))
echo "  Temporal context for streams: $stream_context (-$streams_start_frame_ext:$streams_end_frame_ext)"
echo "stream_context=$stream_context" >> $out_param_file

# number of streams
nstreams=1
while [ "${stream_ipattern[$nstreams]}" != "" ] ; do
        nstreams=$(( $nstreams+1 ))
done
nstreams=$(( nstreams - 1 ))
echo "  Number of streams:            $nstreams" 
echo "nstreams=$nstreams" >> $out_param_file


# substitute variables in stream input patterns
stream_ipatterns_sub=""
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  stream_ipattern_sub[$i]=`echo "${stream_ipattern[$i]}" | awk '{ \
                              gsub("\\\\[NFEATURES\\\\]", "'$nfeatures'"); gsub("\\\\[CONTEXT\\\\]", "'$stream_context'"); \
                              gsub("\\\\[NSTREAMS\\\\]", "'$nstreams'"); print;}'`
                                 
  if [ "$stream_ipatterns_sub" = "" ] ; then
    stream_ipatterns_sub="${stream_ipattern_sub[$i]}"
  else
    stream_ipatterns_sub="$stream_ipatterns_sub | ${stream_ipattern_sub[$i]}"
  fi
  echo "stream_ipattern_sub[$i]=\"${stream_ipattern_sub[i]}\"" >> $out_param_file        
done
echo "  Input patterns for streams:   $stream_ipatterns_sub"

# sizes of streams inputs
stream_isizes=""
stream_nblocks_s=""
streams_total_isize=0
streams_total_nblocks=0
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  v=`$script_dir/misc/getnvals.pl ${stream_ipattern_sub[$i]}`
  stream_isize[i]=`echo $v | awk '{print $1;}'`    # number of input features for the stream
  stream_nblocks[i]=`echo $v | awk '{print $2;}'`  # number of rectangular blocks in the input pattern
  streams_total_isize=$(( streams_total_isize + stream_isize[i] ))
  streams_total_nblocks=$(( streams_total_nblocks + stream_nblocks[i] ))
  if [ "$stream_isizes" = "" ] ; then
    stream_isizes="${stream_isize[i]}"
    stream_nblocks_s="${stream_nblocks[i]}"
  else
    stream_isizes="$stream_isizes ${stream_isize[i]}"
    stream_nblocks_s="$stream_nblocks_s ${stream_nblocks[i]}"
  fi
  echo "stream_isize[$i]=${stream_isize[i]}" >> $out_param_file
  echo "stream_nblocks[$i]=${stream_nblocks[i]}" >> $out_param_file
done
echo "  Input sizes for streams:      $stream_isizes"
echo "  Rect. blocks in inp. pattern: $stream_nblocks_s"
echo "  Total input size for streams: $streams_total_isize"
echo "  Total input blocks for str.:  $streams_total_nblocks"
echo "streams_total_isize=$streams_total_isize" >> $out_param_file
echo "streams_total_nblocks=$streams_total_nblocks" >> $out_param_file


# substitute variables in stream input windows
stream_iwindows=""
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  stream_iwindow_sub[$i]=`echo "${stream_iwindow[$i]}" | awk '{ \
                            gsub("\\\\[NFEATURES\\\\]", "'$nfeatures'"); gsub("\\\\[CONTEXT\\\\]", "'$stream_context'"); \
                            gsub("\\\\[NSTREAMS\\\\]", "'$nstreams'"); gsub("\\\\[INPUT_SIZE\\\\]", "'"${stream_isize[$i]}"'");  print;}'`
  if [ "$stream_iwindows" = "" ] ; then
    stream_iwindows="${stream_iwindow_sub[$i]}"
  else
    stream_iwindows="$stream_iwindows | ${stream_iwindow_sub[$i]}"
  fi
  echo "stream_iwindow_sub[$i]=\"${stream_iwindow_sub[i]}\"" >> $out_param_file        
done
echo "  Stream input windows:         $stream_iwindows"


# substitute variables in stream input transforms
stream_itransforms=""
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  stream_itransform_sub[$i]=`echo "${stream_itransform[$i]}" | awk '{ \
                               gsub("\\\\[NFEATURES\\\\]", "'$nfeatures'"); gsub("\\\\[CONTEXT\\\\]", "'$stream_context'"); \
                               gsub("\\\\[NSTREAMS\\\\]", "'$nstreams'"); gsub("\\\\[INPUT_SIZE\\\\]", "'"${stream_isize[$i]}"'"); \
                               gsub("{{.+}}", "");  print;}' | $script_dir/misc/expand_tr.pl | tr '\n' ' ' | awk '{gsub(" +", " "); print;}'`                               
  if [ "$stream_itransforms" = "" ] ; then
    stream_itransforms="${stream_itransform_sub[$i]}"
  else
    stream_itransforms="$stream_itransforms | ${stream_itransform_sub[$i]}"
  fi
  echo "stream_itransform_sub[$i]=\"${stream_itransform_sub[i]}\"" >> $out_param_file        
done
echo "  Stream input transform:       $stream_itransforms"


# get stream neural network input vector sizes
stream_net_isizes=""
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  stream_net_isize[$i]=`$script_dir/misc/tr_osize.pl "${stream_itransform_sub[$i]}"`
  if [ "$stream_net_isizes" = "" ] ; then
    stream_net_isizes="${stream_net_isize[$i]}"
  else
    stream_net_isizes="$stream_net_isizes ${stream_net_isize[$i]}"
  fi
  echo "stream_net_isize[$i]=${stream_net_isize[$i]}" >> $out_param_file        
done
echo "  Stream NN input sizes:        $stream_net_isizes"


# substitute variables in definitions of neural nerural networks
stream_net_nlayers_s=""
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  stream_net_sub[$i]=`echo "${stream_net[$i]}" | awk '{gsub("\\\\[INPUT_SIZE\\\\]", "'"${stream_net_isize[$i]}"'"); gsub("\\\\[OUTPUT_SIZE\\\\]", "'$nclasses'"); print}' \
    | tr ';' '\n' | awk '! /^ *$/ {gsub("^ +", ""); print};'`
  stream_net_nlayers[$i]=`echo "${stream_net_sub[$i]}" | wc | awk '{print $1;}'`
  if [ "${stream_net_sub[$i]}" = "" ] ; then
    stream_net_nlayers[$i]=0
  fi
  if [ "$stream_net_nlayers_s" = "" ] ; then
    stream_net_nlayers_s="${stream_net_nlayers[$i]}"
  else
    stream_net_nlayers_s="$stream_net_nlayers_s ${stream_net_nlayers[$i]}"
  fi
  echo "stream_net_sub[$i]=\"${stream_net_sub[i]}\"" >> $out_param_file          
  echo "stream_net_nlayers[$i]=${stream_net_nlayers[i]}" >> $out_param_file            
done
echo "  Stream NN numbers of layers:  $stream_net_nlayers_s"


# get final number of layers in cut stream neural networks; get stream neural network output vector sizes
stream_nused_layers_s=""
stream_net_osize_s=""
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  if [ ${stream_nused_layers[$i]} -gt ${stream_net_nlayers[$i]} ] ; then
    stream_nused_layers[i]=${stream_net_nlayers[$i]} 
  fi
  
  if [ "${stream_nused_layers[$i]}" != "0" ] ; then
    last_layer=`echo "${stream_net_sub[$i]}" | awk 'NR == '${stream_nused_layers[$i]}' {print}'`
    stream_net_osize[$i]=`$script_dir/misc/tr_osize.pl "$last_layer"`
  else
    stream_net_osize[$i]=${stream_net_isize[$i]}
  fi
  if [ "$stream_nused_layers_s" = "" ] ; then
    stream_nused_layers_s="${stream_nused_layers[$i]}"
  else
    stream_nused_layers_s="$stream_nused_layers_s ${stream_nused_layers[$i]}"
  fi
  if [ "$stream_net_osize_s" = "" ] ; then
    stream_net_osize_s="${stream_net_osize[$i]}"
  else
    stream_net_osize_s="$stream_net_osize_s ${stream_net_osize[$i]}"
  fi
 
  echo "stream_nused_layers[$i]=${stream_nused_layers[$i]}" >> $out_param_file          
  echo "stream_net_osize[$i]=${stream_net_osize[$i]}" >> $out_param_file          
done
echo "  Stream NN n. of used layers:  $stream_nused_layers_s"
echo "  Stream NN output sizes:       $stream_net_osize_s"


# substitute variables in stream output transforms
stream_otransforms=""
stream_osizes=""
streams_osize=0;
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  stream_otransform_sub[$i]=`echo "${stream_otransform[$i]}" | awk '{ \
                               gsub("\\\\[NFEATURES\\\\]", "'$nfeatures'"); gsub("\\\\[CONTEXT\\\\]", "'$stream_context'"); \
                               gsub("\\\\[NSTREAMS\\\\]", "'$nstreams'"); gsub("\\\\[INPUT_SIZE\\\\]", "'"${stream_net_osize[$i]}"'"); \
                               gsub("{{.+}}", "");  print;}'`
  stream_osize[$i]=`$script_dir/misc/tr_osize.pl "${stream_otransform_sub[$i]}"`
  if [ "$stream_otransforms" = "" ] ; then
    stream_otransforms="${stream_otransform_sub[$i]}"
  else
    stream_otransforms="$stream_otransforms | ${stream_otransform_sub[$i]}"
  fi
  if [ "$stream_osizes" = "" ] ; then
    stream_osizes="${stream_osize[$i]}"
  else
    stream_osizes="$stream_osizes ${stream_osize[$i]}"
  fi  
  echo "stream_otransform_sub[$i]=\"${stream_otransform_sub[i]}\"" >> $out_param_file
  echo "stream_osize[$i]=${stream_osize[$i]}" >> $out_param_file
  streams_osize=$(( streams_osize + stream_osize[$i]))
done
echo "  Stream output transform:      $stream_otransforms"
echo "  Stream output sizes:          $stream_osizes"
echo "streams_osize=$streams_osize" >> $out_param_file
echo "  Total output size of streams: $streams_osize"


# substitute variables in stream output functions
stream_ofuncs=""
for (( i=1 ; i <= nstreams; i=i+1 )) ; do
  stream_ofunc_sub[$i]=`echo "${stream_ofunc[$i]}" | awk '{ \
                               gsub("\\\\[NFEATURES\\\\]", "'$nfeatures'"); gsub("\\\\[CONTEXT\\\\]", "'$stream_context'"); \
                               gsub("\\\\[NSTREAMS\\\\]", "'$nstreams'"); gsub("\\\\[SIZE\\\\]", "'"${stream_osize[$i]}"'"); \
                               gsub("{{.+}}", "");  print;}'`
  if [ "$stream_ofuncs" = "" ] ; then
    stream_ofuncs="${stream_ofunc_sub[$i]}"
  else
    stream_ofuncs="$stream_ofuncs | ${stream_ofunc_sub[$i]}"
  fi
  echo "stream_ofunc_sub[$i]=\"${stream_ofunc_sub[i]}\"" >> $out_param_file        
done
echo "  Stream onput functions:       $stream_ofuncs"


# temporal context for merger
merger_context=$(( merger_start_frame_ext - streams_start_frame_ext + merger_end_frame_ext - streams_end_frame_ext + 1 ))
echo "merger_context=$merger_context" >> $out_param_file
echo "  Temporal context for merger:  $merger_context (-$merger_start_frame_ext+$streams_start_frame_ext:$merger_end_frame_ext-$streams_end_frame_ext)"

#input pattern for merger
merger_ipattern_sub=`echo "$merger_ipattern" | awk '{ \
                              gsub("\\\\[MERGER_NFEATURES\\\\]", "'$streams_osize'");  \
                              gsub("\\\\[MERGER_CONTEXT\\\\]", "'$merger_context'"); print;}'`                                 
echo "merger_ipattern_sub=\"${merger_ipattern_sub}\"" >> $out_param_file        
echo "  Input patterns for merger:    ${merger_ipattern_sub}"

# size of merger input, number of blocks at the input
v=`$script_dir/misc/getnvals.pl $merger_ipattern_sub`
merger_isize=`echo $v | awk '{print $1;}'`    # number of input features for merger
merger_nblocks=`echo $v | awk '{print $2;}'`  # number of rectangular blocks in the input pattern
echo "merger_isize=$merger_isize" >> $out_param_file
echo "merger_nblocks=$merger_nblocks" >> $out_param_file
echo "  Input size for merger:        $merger_isize"
echo "  Rect. blocks in inp. pattern: $merger_nblocks"

# substitute variables in merger input window
merger_iwindow_sub=`echo "$merger_iwindow" | awk '{ \
                             gsub("\\\\[NFEATURES\\\\]", "'$nfeatures'"); \
                             gsub("\\\\[NSTREAMS\\\\]", "'$nstreams'"); gsub("\\\\[INPUT_SIZE\\\\]", "'"$merger_isize"'");  print;}'`
echo "merger_iwindow_sub=\"$merger_iwindow_sub\"" >> $out_param_file
echo "  Merger input window:          $merger_iwindow_sub"

# substitute variables in merger input transform
merger_itransform_sub=`echo "$merger_itransform" | awk '{ \
                             gsub("\\\\[NFEATURES\\\\]", "'$nfeatures'"); \
                             gsub("\\\\[NSTREAMS\\\\]", "'$nstreams'"); gsub("\\\\[INPUT_SIZE\\\\]", "'"$merger_isize"'");  print;}'`
echo "merger_itransform_sub=\"$merger_itransform_sub\"" >> $out_param_file
echo "  Merger input transform:       $merger_itransform_sub"

# get input size for merger neural network
merger_net_isize=`$script_dir/misc/tr_osize.pl "$merger_itransform_sub"`
echo "merger_net_isize=$merger_net_isize" >> $out_param_file
echo "  Merger neural net input size: $merger_net_isize"

# substitute variables in definition of merger neural network
merger_net_sub=`echo "$merger_net" | awk '{gsub("\\\\[INPUT_SIZE\\\\]", "'"$merger_net_isize"'"); gsub("\\\\[OUTPUT_SIZE\\\\]", "'$nclasses'"); print}' \
    | tr ';' '\n' | awk '! /^ *$/ {gsub("^ +", ""); print};'`
echo "merger_net_sub=\"$merger_net_sub\"" >> $out_param_file          

# number of layers in merger
merger_net_nlayers=`echo "$merger_net_sub" | wc | awk '{print $1;}'`
if [ "$merger_net_sub" = "" ] ; then
  merger_net_nlayers=0
fi
echo "merger_net_nlayers=$merger_net_nlayers" >> $out_param_file          
echo "  Number of layers in merger NN:$merger_net_nlayers"

# substitute variables in merger output function
merger_ofunc_sub=`echo "$merger_ofunc" | awk '{gsub("\\\\[SIZE\\\\]", "'$nclasses'"); gsub("{{.+}}", "");  print;}'`
echo "merger_ofunc_sub=\"$merger_ofunc_sub\"" >> $out_param_file          
echo "  Merger output function:       $merger_ofunc_sub"

chmod ug+x $out_param_file
