#!/usr/bin/perl

# Finds output dimension for a transformation

sub tr_osize
{
  my $macro = $_[0];

  $macro =~ tr/[\n\t]/  /;
  $macro =~ s/ +/ /g;
  $macro =~ s/^ //g;
  $macro =~ s/ $//g;
  my $params;
  my $osize;
  my $tr_name;

  ($tr_name, $params) = split /\s+/, $macro, 2;
  $tr_name =~ tr/a-z/A-Z/;

  if($tr_name eq "<WINDOW>")                # <WINDOW> size vals
  {
    ($osize, $other) = split /\s/, $params;
  }
  elsif($tr_name eq "<WINDOWPREDEF>")       # <WINDOWPREDEF> size name params
  { 
    ($osize, $other) = split /\s/, $params;
  }
  elsif($tr_name eq "<BIAS>")              # <BIAS> size vals
  {
    ($osize, $other) = split /\s/, $params;
  }
  elsif($tr_name eq "<BIASPREDEF>")        # <BIASPREDEF> size name params
  {
    ($osize, $other) = split /\s/, $params;
  }
  elsif($tr_name eq "<SOFTMAX>")           # <SOFTMAX> size
  {
    $osize = $params;
  }
  elsif($tr_name eq "<LOG>")               # <LOG> size
  {
    $osize = $params;
  }
  elsif($tr_name eq "<SIGMOID>")           # <SIGMOID> size
  {
    $osize = $params;
  }
  elsif($tr_name eq "<XFORM>")             # <XFORM> out_size in_size vals
  {
    ($osize, $other) = split /\s/, $params;
  }
  elsif($tr_name eq "<XFORMPREDEF>")       # <XFORMPREDEF> out_size in_size name params
  {
    ($osize, $other) = split /\s/, $params;
  }
  elsif($transform =~ /{{.+}}/)
  {
    $transform =~ /{{(.+)}}/;              # user can specify the output size manually in {{ }} 
    $osize = $1;
  }
  elsif($tr_name eq "<NUMBLOCKS>")
  {
    $osize = 0;
    ($nblocks, $params) = split /\s+/, $params, 2;

    my @trans = split /<Block>/, $params;
    my $tr;

    foreach $tr (@trans)
    {
      if($tr ne "")
      {
        $tr =~ s/^ +//;
        ($n, $tr) = split /\s+/, $tr, 2;
        $osize += tr_osize($tr);
      }
    }
  }
  else
  {
    print STDERR "ERROR: Can not determine output size of transform: '$tr'\n";
    exit 1;
  }
  return $osize;
}

if($ARGV[0] eq "")
{
	print "SYNTAX: tr_osize.pl transform\n";
	exit(1);
}

$transform = $ARGV[0];
$osize = tr_osize($transform);

print "$osize\n";

