#!/bin/sh

if [ $# -ne 1 ] ; then
  echo USAGE: train_stream.sh config_file
  exit 1
fi

conf=$1

# configuration
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf
out_dir=`echo "$out_dir" | awk --posix '/\/it[0-9]{3}*/ {print; next;} {print $1 "/it001";}'`
source $out_dir/config/params.sh

echo "Training merger ..."

# check if the neural network does not exist
if [ -e $out_dir/weights/merger/final/merger_nn.ftr ] ; then
  exit 0
fi

# forward data through lower neural networks and save it (if enabled)
if [ $save_streams_output = "1" ] ; then
  echo "  Forwarding data through lower neural networks ..."
    
  mkdir -p $main_work_dir/merger_input
  mkdir -p $out_dir/lists
    
  ${script_dir}/misc/filterflist.sh $out_dir/lists/train_fea.scp 1 0 $main_work_dir/merger_input > $out_dir/lists/train_fea_flt.scp
  $bin_dir/SFeaCat -T 1 \
                   -H $out_dir/weights/streams_fwd.ftr \
                   -S $out_dir/lists/train_fea_flt.scp \
                   -l $main_work_dir/merger_input \
                   --start-frm-ext=$streams_start_frame_ext \
                   --end-frm-ext=$streams_end_frame_ext

  mkdir -p $main_work_dir/merger_input
  ${script_dir}/misc/filterflist.sh $out_dir/lists/cv_fea.scp 1 0 $main_work_dir/merger_input > $out_dir/lists/cv_fea_flt.scp
  $bin_dir/SFeaCat -T 1 \
                   -H $out_dir/weights/streams_fwd.ftr \
                   -S $out_dir/lists/cv_fea_flt.scp \
                   -l $main_work_dir/merger_input \
                   --start-frm-ext=$streams_start_frame_ext \
                   --end-frm-ext=$streams_end_frame_ext                     

  mkdir -p $main_work_dir/merger_input
  ${script_dir}/misc/filterflist.sh $out_dir/lists/test_fea.scp 1 0 $main_work_dir/merger_input > $out_dir/lists/test_fea_flt.scp
  $bin_dir/SFeaCat -T 1 \
                   -H $out_dir/weights/streams_fwd.ftr \
                   -S $out_dir/lists/test_fea_flt.scp \
                   -l $main_work_dir/merger_input \
                   --start-frm-ext=$streams_start_frame_ext \
                   --end-frm-ext=$streams_end_frame_ext                                          
  
  awk -F '=' '{gsub("^.*/", "", $1); gsub("\\.[^.]*$", "", $1); print "'$main_work_dir'/merger_input/" $1 ".fea";}' $out_dir/lists/train_fea.scp > $out_dir/lists/train_spost.scp
  awk -F '=' '{gsub("^.*/", "", $1); gsub("\\.[^.]*$", "", $1); print "'$main_work_dir'/merger_input/" $1 ".fea";}' $out_dir/lists/cv_fea.scp > $out_dir/lists/cv_spost.scp
  awk -F '=' '{gsub("^.*/", "", $1); gsub("\\.[^.]*$", "", $1); print "'$main_work_dir'/merger_input/" $1 ".fea";}' $out_dir/lists/test_fea.scp > $out_dir/lists/test_spost.scp
  awk -F '=' '{gsub("^.*/", "", $1); gsub("\\.[^.]*$", "", $1); print "'$main_work_dir'/merger_input/" $1 ".fea";}' $out_dir/lists/lre_fea.scp > $out_dir/lists/lre_spost.scp
fi 

# estimation of normalization coefficients (means and variances) using single state HMM
mkdir -p $out_dir/tmp
echo "norm_est" > $out_dir/tmp/merger_norm.net

if [ ! -e $out_dir/norms/merger_norm_est.hmmdefs ] ; then
  echo "  Estimation of normalization coefs ..."
  mkdir -p $out_dir/norms
  if [ $save_streams_output = "1" ] ; then
    # read pregenerated features for merger
    if ! ${bin_dir}/SERest -T ${trace_streams_normest:=0} \
                      -H $out_dir/macros/merger_train_norm_posti.ftr \
                      -H $out_dir/macros/merger_norm_est.hmmdefs \
                      -S $out_dir/lists/train_spost.scp \
                      -M $out_dir/norms \
                      -G HTK \
                      -D \
                      --start-frm-ext=$(( merger_start_frame_ext - streams_start_frame_ext )) \
                      --end-frm-ext=$(( merger_end_frame_ext - streams_end_frame_ext )) \
                      --recog-net=$out_dir/tmp/merger_norm.net ; then
      exit 1
    fi
    rm -f $out_dir/norms/merger_train_norm_posti.ftr
  else
    # read input features
    if ! ${bin_dir}/SERest -T ${trace_streams_normest:=0} \
                      -H $out_dir/weights/streams.ftr \
                      -H $out_dir/macros/merger_train_norm.ftr \
                      -H $out_dir/macros/merger_norm_est.hmmdefs \
                      -S $out_dir/lists/train_fea.scp \
                      -M $out_dir/norms \
                      -G HTK \
                      -D \
                      --start-frm-ext=$merger_start_frame_ext \
                      --end-frm-ext=$merger_end_frame_ext \
                      --recog-net=$out_dir/tmp/merger_norm.net ; then
      exit 1
    fi
    rm -f $out_dir/norms/merger_train_norm.ftr
    rm -f $out_dir/norms/streams.ftr                      
  fi
fi

# conversion of the HMM to a transform
if [ ! -e $out_dir/norms/merger_norm.ftr ] ; then
  echo "  Converting HMM file to normalization macro ..."
  mean=`awk '/<Mean>/ {getline; for(i=1; i<=NF; i++) printf " %e", -1 * $i;}' $out_dir/norms/merger_norm_est.hmmdefs`
  var_1=`awk '/<Variance>/ {getline; for(i=1; i<=NF; i++) printf " %e", 1 / sqrt($i);}' $out_dir/norms/merger_norm_est.hmmdefs`    
    
  nn_isize=`echo "$mean" | wc | awk '{print $2}'`
  
  awk '{gsub("\\[NORM_NAME\\]", "merger_norm");
        gsub("\\[NET_ISIZE\\]", "'$nn_isize'");
        gsub("\\[BIAS_VALS\\]", "'"$mean"'");
	gsub("\\[WINDOW_VALS\\]", "'"$var_1"'");
	print;}' ${script_dir}/templ/norm.ftr > $out_dir/norms/merger_norm.ftr
fi

# neural network training
if [ ! -e $out_dir/weights/merger/final/merger_nn.ftr ] ; then
  if [ $save_streams_output = "1" ] ; then
    if ! ${script_dir}/train_nn.sh $conf \
                  --crossvalidation=FALSE \
                  --cache-size=${net_train_cache_size:=16000} \
                  --bunch-size=${net_train_bunch_size:=16} \
                  --learning-rate=${merger_learning_rate:=0.008} \
                  --randomize=${net_train_randomize:=TRUE} \
                  --seed=${net_train_seed:=1000} \
                  --clients=${net_train_clients:=0} \
                  --synchronize=${net_train_synchronize:=TRUE} \
                  --port=${net_train_port:=2020} \
                  --ncumrbu=${net_train_ncumrbu:=0} \
                  --output-label-map=$out_dir/dicts/dict \
                  --start-frm-ext=$(( merger_start_frame_ext - streams_start_frame_ext )) \
                  --end-frm-ext=$(( merger_end_frame_ext - streams_end_frame_ext )) \
                  --source-mlf=$out_dir/mlfs/ref.mlf \
                  --source-mmf=$out_dir/norms/merger_norm.ftr \
                  --source-mmf=$out_dir/macros/merger_nn.ftr \
                  --source-mmf=$out_dir/macros/merger_train_nn_posti.ftr \
                  --print-config=TRUE \
                  --trace=1 \
                  --target-model-dir=$out_dir/weights/merger/final \
                  --train-dir=$out_dir/weights/merger \
                  --script=$out_dir/lists/train_spost.scp \
                  --cv-script=$out_dir/lists/cv_spost.scp \
                  --lr-estim-script=$out_dir/lists/lre_spost.scp \
                  --ts-min-iter=${ts_min_iter:=5} \
                  --ts-max-iter=${ts_max_iter:=9999} \
                  --ts-delta-stop=${ts_delta_stop:=.5} \
                  --ts-delta-dec=${ts_delta_dec:=.5} \
                  --ts-keep-dec=${ts_keep_dec:=1} ; then
      exit 1
    fi
  else                  
    if ! ${script_dir}/train_nn.sh $conf \
                  --crossvalidation=FALSE \
                  --cache-size=${net_train_cache_size:=16000} \
                  --bunch-size=${net_train_bunch_size:=16} \
                  --learning-rate=${merger_learning_rate:=0.008} \
                  --randomize=${net_train_randomize:=TRUE} \
                  --seed=${net_train_seed:=0} \
                  --clients=${net_train_clients:=0} \
                  --synchronize=${net_train_synchronize:=TRUE} \
                  --port=${net_train_port:=2020} \
                  --ncumrbu=${net_train_ncumrbu:=0} \
                  --output-label-map=$out_dir/dicts/dict \
                  --start-frm-ext=$streams_start_frame_ext \
                  --end-frm-ext=$streams_end_frame_ext \
                  --source-mlf=$out_dir/mlfs/ref.mlf \
                  --source-mmf=$out_dir/norms/merger_norm.ftr \
                  --source-mmf=$out_dir/weights/streams.ftr \
                  --source-mmf=$out_dir/macros/merger_nn.ftr \
                  --source-mmf=$out_dir/macros/merger_train_nn.ftr \
                  --print-config=TRUE \
                  --trace=1 \
                  --target-model-dir=$out_dir/weights/merger/final \
                  --train-dir=$out_dir/weights/merger \
                  --script=$out_dir/lists/train_fea.scp \
                  --cv-script=$out_dir/lists/cv_fea.scp \
                  --lr-estim-script=$out_dir/lists/lre_fea.scp \
                  --ts-min-iter=${ts_min_iter:=5} \
                  --ts-max-iter=${ts_max_iter:=9999} \
                  --ts-delta-stop=${ts_delta_stop:=.5} \
                  --ts-delta-dec=${ts_delta_dec:=.5} \
                  --ts-keep-dec=${ts_keep_dec:=1} ; then
      exit 1
    fi
  fi                                    
fi

# cut off some final layers if specified
#if [ ! -e $out_dir/weights/stream${stream_idx}/final/stream${stream_idx}_nn_cut.ftr ] ; then
#  ${script_dir}/misc/cutoff_layers.pl $out_dir/weights/stream${stream_idx}/final/stream${stream_idx}_nn.ftr ${stream_nused_layers[$stream_idx]} > $out_dir/weights/stream${stream_idx}/final/stream${stream_idx}_nn_cut.ftr
#fi

if [ -e $main_work_dir/merger_input ] && [ ! "$keep_tmp_files" = "1" ] ; then
  rm -f -r $main_work_dir/merger_input
fi
