#!/bin/sh

if [ $# -ne 2 ] ; then
	echo USAGE: train_stream.sh config_file stream{1-N}
	exit 1
fi

conf=$1
stream_idx=$2

# configuration
conf_path=${conf%\/*}
conf_global="$conf_path/_global.sh"

source $conf_global
source $conf
out_dir=`echo "$out_dir" | awk --posix '/\/it[0-9]{3}*/ {print; next;} {print $1 "/it001";}'`
source $out_dir/config/params.sh

echo "Training stream $stream_idx ..."

# no neural network for this stream
if [ "${stream_nused_layers[$stream_idx]}" == "0" ] ; then
  echo "No neural network associated."
  exit 0
fi

# estimation of normalization coefficients (means and variances) using single state HMM
mkdir -p $out_dir/tmp
echo "norm_est" > $out_dir/tmp/stream${stream_idx}_norm.net

if [ ! -e $out_dir/norms/stream${stream_idx}_norm_est.hmmdefs ] ; then
  echo "  Estimation of normalization coefs ..."
  mkdir -p $out_dir/norms
  ${bin_dir}/SERest -T ${trace_streams_normest:=0} \
                    -H $out_dir/macros/input_mat.ftr \
                    -H $out_dir/macros/stream${stream_idx}_train_norm.ftr \
                    -H $out_dir/macros/stream${stream_idx}_norm_est.hmmdefs \
                    -S $out_dir/lists/train_fea.scp \
                    -M $out_dir/norms \
                    -G HTK \
                    -D \
                    --start-frm-ext=$streams_start_frame_ext \
                    --end-frm-ext=$streams_end_frame_ext \
                    --recog-net=$out_dir/tmp/stream${stream_idx}_norm.net
  rm -f $out_dir/norms/stream${stream_idx}_train_norm.ftr
  rm -f $out_dir/norms/input_mat.ftr
fi

# conversion of the HMM to a transform
if [ ! -e $out_dir/norms/stream${stream_idx}_norm.ftr ] ; then
  echo "  Converting HMM file to normalization macro ..."
  mean=`awk '/<Mean>/ {getline; for(i=1; i<=NF; i++) printf " %e", -1 * $i;}' $out_dir/norms/stream${stream_idx}_norm_est.hmmdefs`
  var_1=`awk '/<Variance>/ {getline; for(i=1; i<=NF; i++) printf " %e", 1 / sqrt($i);}' $out_dir/norms/stream${stream_idx}_norm_est.hmmdefs`
  
  nn_isize=`echo "$mean" | wc | awk '{print $2}'`
  
  awk '{gsub("\\[NORM_NAME\\]", "stream'${stream_idx}'_norm");
        gsub("\\[NET_ISIZE\\]", "'$nn_isize'");
        gsub("\\[BIAS_VALS\\]", "'"$mean"'");
	gsub("\\[WINDOW_VALS\\]", "'"$var_1"'");
	print;}' ${script_dir}/templ/norm.ftr > $out_dir/norms/stream${stream_idx}_norm.ftr
fi

# neural network training
if [ ! -e $out_dir/weights/stream${stream_idx}/final/stream${stream_idx}_nn.ftr ] ; then
 
  if !  ${script_dir}/train_nn.sh $conf \
                  --crossvalidation=FALSE \
                  --cache-size=${net_train_cache_size:=16000} \
                  --bunch-size=${net_train_bunch_size:=16} \
                  --learning-rate=${streams_learning_rate:=0.008} \
                  --randomize=${net_train_randomize:=TRUE} \
                  --seed=${net_train_seed:=1000} \
                  --clients=${net_train_clients:=0} \
                  --synchronize=${net_train_synchronize:=TRUE} \
                  --port=${net_train_port:=2020} \
                  --ncumrbu=${net_train_ncumrbu:=0} \
                  --output-label-map=$out_dir/dicts/dict \
                  --start-frm-ext=$streams_start_frame_ext \
                  --end-frm-ext=$streams_end_frame_ext \
                  --source-mlf=$out_dir/mlfs/ref.mlf \
                  --source-mmf=$out_dir/macros/input_mat.ftr \
                  --source-mmf=$out_dir/norms/stream${stream_idx}_norm.ftr \
                  --source-mmf=$out_dir/macros/stream${stream_idx}_nn.ftr \
                  --source-mmf=$out_dir/macros/stream${stream_idx}_train_nn.ftr \
                  --print-config=TRUE \
                  --trace=1 \
                  --target-model-dir=$out_dir/weights/stream${stream_idx}/final \
                  --train-dir=$out_dir/weights/stream${stream_idx} \
                  --script=$out_dir/lists/train_fea.scp \
                  --cv-script=$out_dir/lists/cv_fea.scp \
                  --lr-estim-script=$out_dir/lists/lre_fea.scp \
                  --ts-min-iter=${ts_min_iter:=5} \
                  --ts-max-iter=${ts_max_iter:=9999} \
                  --ts-delta-stop=${ts_delta_stop:=.5} \
                  --ts-delta-dec=${ts_delta_dec:=.5} \
                  --ts-keep-dec=${ts_keep_dec:=1} ; then
     exit 1
   fi		  
fi

# cut off some final layers if specified
if [ ! -e $out_dir/weights/stream${stream_idx}/final/stream${stream_idx}_nn_cut.ftr ] ; then
  ${script_dir}/misc/cutoff_layers.pl $out_dir/weights/stream${stream_idx}/final/stream${stream_idx}_nn.ftr ${stream_nused_layers[$stream_idx]} > $out_dir/weights/stream${stream_idx}/final/stream${stream_idx}_nn_cut.ftr
fi
